/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "CLucene/StdHeader.h"
#include "KoreanAnalyzer.h"
#include "KoreanTokenizer.h"
#include "KoreanFilter.h"

CL_NS_USE(analysis)
CL_NS_USE(util)
CL_NS_DEF2(analysis, kr)

const TCHAR** KoreanAnalyzer::STOP_WORDS = CL_NS(analysis)::StopAnalyzer::ENGLISH_STOP_WORDS;
const TCHAR* KoreanAnalyzer::KOR_STOP_WORDS[] = {
	_T("이"), _T("그"), _T("저"), _T("것"), _T("수"), _T("등"), _T("들")
};
const TCHAR* KoreanAnalyzer::DIC_ENCODING = _T("UTF-8");


KoreanAnalyzer::KoreanAnalyzer(const TCHAR** stopWords):
	bigrammable(true), stopSet(false)
{
	StopFilter::fillStopTable(&stopSet, stopWords);
	StopFilter::fillStopTable(&stopSet, KOR_STOP_WORDS);
}
/*
KoreanAnalyzer::KoreanAnalyzer(File stopwords)
{
	InputStream is = new FileInputStream(stopwords);
	InputStreamReader reader = new InputStreamReader(is, DIC_ENCODING);		
	stopSet = WordlistLoader.getWordSet(reader);
}


KoreanAnalyzer::KoreanAnalyzer(File stopwords, String encoding)
{
	InputStream is = new FileInputStream(stopwords);
	InputStreamReader reader = new InputStreamReader(is, encoding);		
	stopSet = WordlistLoader.getWordSet(reader);
}
		

KoreanAnalyzer::KoreanAnalyzer(Reader stopwords)
{
	stopSet = WordlistLoader.getWordSet(stopwords);
}
*/
KoreanAnalyzer::~KoreanAnalyzer()
{
}

TokenStream* KoreanAnalyzer::tokenStream(const TCHAR* fieldName, Reader* reader)
{
	KoreanTokenizer *tokenStream = _CLNEW KoreanTokenizer(reader);
	TokenStream *result = _CLNEW KoreanFilter(tokenStream, bigrammable, true);
	result = _CLNEW LowerCaseFilter(result, true);
	result = _CLNEW StopFilter(result, true, &stopSet);
	return result;
}
	
void KoreanAnalyzer::setBigrammable(bool is)
{
	bigrammable = is;
}

CL_NS_END2
