/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_KoreanAnalyzer
#define _lucene_analysis_kr_KoreanAnalyzer

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/util/Reader.h"
#include "CLucene/analysis/Analyzers.h"
#include "morph/MorphException.h"

CL_NS_DEF2(analysis, kr)

/**
 * Filters {@link StandardTokenizer} with {@link StandardFilter}, {@link
 * LowerCaseFilter} and {@link StopFilter}, using a list of English stop words.
 *
 * @version $Id: KoreanAnalyzer.java,v 1.1 2008/08/26 13:26:48 dev01 Exp $
 */

class KoreanAnalyzer : public Analyzer
{
private:
	CL_NS(util)::CLSetList<const TCHAR*> stopSet;  
	bool bigrammable;

public:
	/** An array containing some common English words that are usually not
		useful for searching. */
	static const TCHAR** STOP_WORDS;
	static const TCHAR* KOR_STOP_WORDS[];
	static const TCHAR* DIC_ENCODING;

	/** Default maximum allowed token length */
    static const int DEFAULT_MAX_TOKEN_LENGTH = 255;
	  
	KoreanAnalyzer(const TCHAR** stopWords = STOP_WORDS);

	/** Builds an analyzer with the stop words from the given file.
	 *  @see WordlistLoader#getWordSet(File)
	 */
	//KoreanAnalyzer(File stopwords);
	
	/** Builds an analyzer with the stop words from the given file.
	 *  @see WordlistLoader#getWordSet(File)
	 */
	//KoreanAnalyzer(File stopwords, String encoding);
		
	/** Builds an analyzer with the stop words from the given reader.
	 *  @see WordlistLoader#getWordSet(Reader)
	 */
    //KoreanAnalyzer(Reader stopwords);

	~KoreanAnalyzer();
	
	TokenStream* tokenStream(const TCHAR* fieldName, CL_NS(util)::Reader* reader);
	
	/**
	 * determine whether the bigram index term is returned or not if a input word is failed to analysis
	 * If true is set, the bigram index term is returned. If false is set, the bigram index term is not returned.
	 * @param is
	 */
	void setBigrammable(bool is);
};

CL_NS_END2
#endif
