/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_KoreanFilter
#define _lucene_analysis_kr_KoreanFilter

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/StdHeader.h"
#include "CLucene/analysis/Analyzers.h"
#include "morph/MorphAnalyzer.h"
#include "morph/MorphException.h"
#include "utils/String.h"
#include <ext/hash_map>

CL_NS_DEF2(analysis, kr)

class KoreanFilter : public TokenFilter {
private:
	struct StringHash {
		// FIXME: overflow? (wchar_t may be 4bytes)
		size_t operator()(const String& _Key) const	{
			unsigned long __h = 0;
			for (int i = 0 ; i < _Key.length(); i++)
				__h = 5 * __h + _Key[i];
			return size_t(__h);
		}
		
	};
	
	struct StringEqual {
		bool operator()(const String& s1, const String& s2) const {
			return (s1.compare(s2) == 0) ? true : false;
		}
	};	
	typedef __gnu_cxx::hash_map<String, int, StringHash, StringEqual> HashMap;

private:
	CL_NS(util)::CLLinkedList<Token*, CL_NS(util)::Deletor::Void<Token> > koreanQueue;
	
	MorphAnalyzer morph;
	
	bool bigrammable;
	
	static const TCHAR* APOSTROPHE_TYPE;
	static const TCHAR* ACRONYM_TYPE;

public:
	KoreanFilter(TokenStream *input, bool deleteTokenStream);

	/**
	 * 
	 * @param input	input token stream
	 * @param bigramg	Whether the bigram index term return or not.
	 */
	KoreanFilter(TokenStream *input, bool bigram, bool deleteTokenStream);
	
	bool next(Token *token) throw(MorphException);
	
private:
	bool analysisKorean(Token *token, int skipinc) throw(MorphException);
	
	void addBiagramToMap(const String& input, HashMap& map);
	
	String findAlphaNumeric(const String& text);
	
	bool analysisETC(Token* t) throw(MorphException);
	
	bool isAlphaNumChar(TCHAR c);
};


CL_NS_END2
#endif
