/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_KoreanTokenizer
#define _lucene_analysis_kr_KoreanTokenizer

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/analysis/AnalysisHeader.h"
#include "CLucene/util/Reader.h"
#include "KoreanAnalyzer.h"

CL_NS_DEF2(analysis, kr)

class KoreanTokenizer : public Tokenizer
{
public:
	static const int ALPHANUM          = 0;
	static const int APOSTROPHE        = 1;
	static const int ACRONYM           = 2;
	static const int COMPANY           = 3;
	static const int EMAIL             = 4;
	static const int HOST              = 5;
	static const int NUM               = 6;
	static const int CJ                = 7;
	static const int ACRONYM_DEP       = 8;
	static const int KOREAN            = 9;

	/** String token types that correspond to token type int constants */
	static const TCHAR* TOKEN_TYPES[];

private:
	static const int maxTokenLength = KoreanAnalyzer::DEFAULT_MAX_TOKEN_LENGTH;
	static const int DEFAULT_BUFLEN = 16384;
	
	TCHAR*	limit;
	TCHAR*	start;
	TCHAR*	cursor;
	TCHAR*	marker;	
	
	TCHAR*	buffer;
	TCHAR*	bufferEnd;
	int32_t	bufferLength;
	int32_t	bufferOffset;  /* buffer offset in the source text */
	
	TCHAR	tokenBuffer[maxTokenLength+1];

public:
	KoreanTokenizer(CL_NS(util)::Reader* input);
	~KoreanTokenizer();
	
	bool next(Token* token);

private:
	inline void yyfill();	
};

CL_NS_END2
#endif
