/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_UnmodifiableIterator
#define _lucene_analysis_kr_UnmodifiableIterator

CL_NS_DEF2(analysis, kr)

class UnsupportedOperationException {
private:
	char *message;

public:
	UnsupportedOperationException(): message(NULL) {}	
	UnsupportedOperationException(const char *msg) {
		message = strdup(msg);
	}

	~UnsupportedOperationException() { free(message); }
	
	const char* getMessage() const {
		return message;
	}
};

template<typename E>
class UnmodifiableIterator {
public:
	bool hasNext() {}
	E& next() {}
	void remove() {
		throw UnsupportedOperationException("Cannot remove from this iterator");
	}
};

CL_NS_END2
#endif
