/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "AnalysisOutput.h"
#include "../utils/MorphUtil.h"
#include "PatternConstants.h"

CL_NS_DEF2(analysis, kr)

String AnalysisOutput::toString() {

	String buff;
	
	buff.append(MorphUtil::buildTypeString(this->getStem(),this->getPos()));
	if(!this->getNsfx().empty())
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getNsfx(),PatternConstants::POS_SFX_N));
		
	if(this->getPatn()==PatternConstants::PTN_N||
	   this->getPatn()==PatternConstants::PTN_AID) {

	}else if(this->getPatn()==PatternConstants::PTN_NJ) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));
	}else if(this->getPatn()==PatternConstants::PTN_NSM) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getVsfx(),PatternConstants::POS_SFX_V));
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));			
	}else if(this->getPatn()==PatternConstants::PTN_NSMJ) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getVsfx(),PatternConstants::POS_SFX_V));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_NEOMI));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));
	}else if(this->getPatn()==PatternConstants::PTN_NSMXM) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getVsfx(),PatternConstants::POS_SFX_V));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_COPULA));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getXverb(),PatternConstants::POS_XVERB));		
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));
	}else if(this->getPatn()==PatternConstants::PTN_NJCM) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_SFX_V));
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));	
	}else if(this->getPatn()==PatternConstants::PTN_VM) {
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));				
	}else if(this->getPatn()==PatternConstants::PTN_VMJ) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_NEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));				
	}else if(this->getPatn()==PatternConstants::PTN_VMCM) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_NEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(1),PatternConstants::POS_SFX_N));			
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));				
	}else if(this->getPatn()==PatternConstants::PTN_VMXM) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_COPULA));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getXverb(),PatternConstants::POS_XVERB));
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getEomi(),PatternConstants::POS_EOMI));				
	}else if(this->getPatn()==PatternConstants::PTN_VMXMJ) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(1),PatternConstants::POS_COPULA));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getXverb(),PatternConstants::POS_XVERB));	
		if(!this->getPomi().empty()) 
			buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getPomi(),PatternConstants::POS_PEOMI));	
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->elist.at(0),PatternConstants::POS_NEOMI));			
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));				
	}else if(this->getPatn()==PatternConstants::PTN_ADVJ) {
		buff.append(_T(",")).append(MorphUtil::buildTypeString(this->getJosa(),PatternConstants::POS_JOSA));				
	}
	return buff;
}

CL_NS_END2
