/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_morph_AnalysisOutput
#define _lucene_analysis_kr_morph_AnalysisOutput

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/StdHeader.h"
#include "CompoundEntry.h"
#include "../utils/String.h"

CL_NS_DEF2(analysis, kr)

class AnalysisOutput {	
public:		
	static const int SCORE_CORRECT = 100;
	static const int SCORE_ANALYSIS = 30;	
	static const int SCORE_CANDIDATE = 10;
	static const int SCORE_FAIL = 0;

private:
	int score; // score of this result
	int patn; // word pattern
	TCHAR type; // type of input word
	CompoundEntryList compound; // compound noun of input word
	String stem;
	TCHAR pos; // 3 simplified stem type
	TCHAR pos2; // pos attr. for 'pos'
	TCHAR dinf; // pos info. in Han-dic
	String nsfx; // index of noun suffix
	String josa; // josa string
	StringList jlist; // unit-josa sequnce
	String eomi;  // Eomi string
	StringList elist; // unit-Eomi sequence
	String pomi; // prefinal Eomi
	String xverb; // Xverb string
	String vsfx; // verb suffix
	TCHAR vtype; // irregular type

public:
	AnalysisOutput() {
		this->score = SCORE_FAIL;
	}

	AnalysisOutput(const String& stem, const String& josa, const String& eomi, int patn) {
		this->score = SCORE_ANALYSIS;		
		this->stem=stem;
		this->josa = josa;
		this->eomi = eomi;
		this->patn = patn;
	}
	
	AnalysisOutput(const String& stem, const String& josa, const String& eomi, int patn, int socre) {
		this->stem=stem;
		this->josa = josa;
		this->eomi = eomi;
		this->patn = patn;
		this->score = score;
	}
	
	void setScore(int i) {
		this->score = i;
	}
	void setPatn(int i) {
		this->patn = i;
	}
	void setType(TCHAR c) {
		this->type = c;
	}
	
	void setStem(const String& s) {
		this->stem = s;
	}

	void setPos(TCHAR c) {
		this->pos = c;
	}
	
	void setPos2(TCHAR c){
		this->pos2 = c;
	}
	
	void setDinf(TCHAR c){
		this->dinf = c;
	}
	
	void setNsfx(const String& s) {
		this->nsfx = s;		
	}
	
	void setJosa(const String& s) {
		this->josa = s;
	}
	
	void addJlist(const String& l) {
		this->jlist.push_back(l);
	}
	
	void setEomi(const String& s) {
		this->eomi = s;
	}
	
	void addElist(const String& l) {
		this->elist.push_back(l);
	}
		
	void setElist(const String& l, int index){
		this->elist[index] = l;
	}
	
	void setPomi(const String& s) {
		this->pomi = s;
	}
	void setXverb(const String& s){
		this->xverb=s;
	}
	void setVsfx(const String& s) {
		this->vsfx = s;
	}
	void setVtype(TCHAR c) {
		this->vtype = c;
	}

	int getScore() const {
		return this->score;
	}
	int getPatn() const {
		return this->patn;
	}
	
	TCHAR getType() {
		return this->type;
	}	
	const String& getStem() const {
		return stem;
	}	
	TCHAR getPos() {
		return this->pos;
	}
	TCHAR getPos2() {
		return this->pos2;
	}
	TCHAR getDinf() {
		return this->dinf;
	}
	const String& getNsfx() {
		return this->nsfx;
	}
	const String& getJosa() {
		return this->josa;
	}
	const StringList& getJlist() {
		return this->jlist;
	}
	const String& getEomi() {
		return this->eomi;
	}
	const StringList& getElist() {
		return this->elist;
	}
	const String& getPomi(){
		return this->pomi;
	}
	const String& getXverb() {
		return this->xverb;
	}
	const String& getVsfx() {
		return this->vsfx;
	}
	TCHAR getVtype() {
		return this->vtype;
	}
	
	void addCNoun(CompoundEntry w) {
		compound.push_back(w);
	}
	
	CompoundEntryList& getCNounList() {
		return compound;
	}
	
	void setCNoun(const CompoundEntryList& cnoun) {
		compound = cnoun;
	}
	
	void addCNoun(const CompoundEntryList& cnoun) {
		compound.reserve(compound.size()+compound.size());
		compound.insert(compound.end(), cnoun.begin(), cnoun.end());
	}
	
	String toString();
};

struct AnalysisOutputComparator {
	bool operator() (const AnalysisOutput& obj1, const AnalysisOutput& obj2) {
		int score = obj2.getScore()-obj1.getScore();
		int pattern = obj2.getPatn()-obj1.getPatn();
		
		if (score != 0)
			return (score < 0) ? true : false;
		else		
			return (pattern < 0) ? true : false;
	}
};
	
typedef CL_NS(util)::CLVector<AnalysisOutput, CL_NS(util)::Deletor::NullVal<AnalysisOutput> > AnalysisOutputList;

CL_NS_END2
#endif
