/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_morph_CompoundEntry
#define _lucene_analysis_kr_morph_CompoundEntry

#include "CLucene/StdHeader.h"
#include "../utils/String.h"

CL_NS_DEF2(analysis,kr)

/**
 * 복합명사의 개별단어에 대한 정보를 담고있는 클래스 
 * @author S.M.Lee
 *
 */

class CompoundEntry {
private:
	String word;	
	int offset;

public:
	CompoundEntry() {
		offset = -1;
	}
	
	CompoundEntry(const String& w) {
		word = w;
		offset = -1;
	}
	
	CompoundEntry(const String& w,int o) {
		word = w;
		offset = o;		
	}
	
	void setWord(const String& w) {
		word = w;
	}
	
	void setOffset(int o) {
		offset = o;
	}
	
	String getWord() {
		return word;
	}
	
	int getOffset() {
		return offset;
	}
};

typedef CL_NS(util)::CLVector<CompoundEntry,
						CL_NS(util)::Deletor::NullVal<CompoundEntry> > CompoundEntryList;

CL_NS_END2
#endif
