/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_morph_PatternConstant
#define _lucene_analysis_kr_morph_PatternConstant


class PatternConstants {
public:
	/** Hangul word patterns for KMA
	 */
	static const int PTN_N     = 1;   //* 체언 : N/PN/NM/XN/CN/UN/AS/HJ/ET */
	static const int PTN_NJ    = 2;   //* 체언 + 조사 */
	static const int PTN_NSM   = 3;   //* 체언 + 용언화접미사 + 어미 */
	static const int PTN_NSMJ  = 4;   //* 체언 + 용언화접미사 + '음/기' + 조사 */
	static const int PTN_NSMXM = 5;   //* 체언 + 용언화접미사 + '아/어' + 보조용언 + 어미 */
	static const int PTN_NJCM  = 6;   //* 체언 + '에서/부터/에서부터' + '이' + 어미 */

	static const int PTN_VM    = 11;  //* 용언 + 어미 */
	static const int PTN_VMJ   = 12;  //* 용언 + '음/기' + 조사 */
	static const int PTN_VMCM  = 13;  //* 용언 + '음/기' + '이' + 어미 */
	static const int PTN_VMXM  = 14;  //* 용언 + '아/어' + 보조용언 + 어미 */
	static const int PTN_VMXMJ = 15;  //* 용언 + '아/어' + 보조용언 + '음/기' + 조사 */

	static const int PTN_AID   = 21;  //* 단일어 : 부사, 관형사, 감탄사 */
	static const int PTN_ADVJ  = 22;  //* 부사 + 조사 : '빨리도' */

	static const int PTN_NVM   = 31;  //* 체언 + 동사 + 어미 */

	static const int PTN_ZZZ   = 35;  //* 문장부호, KS 완성형 기호열, 단독조사/어미 */	
	
	/**
	 * Definition of sentence types and parts of speech
	 */
	
	//*         CLASSIFICATION OF SENTENCE PATTERNS              */
	static const TCHAR SPTN_DECL = _T('D');       //* declarative sentence */
	static const TCHAR SPTN_QUES =  _T('Q');      //* question sentence    */
	static const TCHAR SPTN_IMPR =  _T('I');       //* imperative sentence  */
	static const TCHAR SPTN_TITL =  _T('T');       //* title of a paragraph */

	//*          CLASSIFICATION OF PARTS OF SPEECH               */
	//	3(basic) + 2(special) types of stem for 'pos'
	static const TCHAR POS_NPXM  =   _T('N');       //* noun, pnoun, xn, nume */
	static const TCHAR POS_VJXV  =   _T('V');       //* verb, adj, xverb      */
	static const TCHAR POS_AID   =   _T('Z');       //* adv, det, excl        */

	static const TCHAR POS_PUNC  =   _T('q');       //* punctuation mark:./,/( */
	static const TCHAR POS_SYMB  =   _T('Q');       //* special symbols       */

	//	normal types of stem for 'pos2'.
	//	Only some of following symbols are used.
	static const TCHAR POS_NOUN  =   _T('N');       //* noun                  */
	static const TCHAR POS_PNOUN  =  _T('P');       //* pronoun               */
	static const TCHAR POS_XNOUN  =  _T('U');       //* dependent noun        */
	static const TCHAR POS_NUMERAL = _T('M');       //* numeral               */

	static const TCHAR POS_PROPER  = _T('O');       //* proper noun: NOT USED */

	static const TCHAR POS_CNOUN  =  _T('C');       //* compound noun guessed */
	static const TCHAR POS_NOUNK  =  _T('u');       //* guessed as noun       */

	static const TCHAR POS_ASCall =  _T('@');       //* all alphanumeric chars*/
	static const TCHAR POS_ASCend =  _T('$');       //* end with alphanumeric */
	static const TCHAR POS_ASCmid =  _T('*');       //* ..+alphanumeric+Hangul*/

	//* defined for numeral to digit conversion */
	static const TCHAR POS_digits =  _T('1');       //* digit-string */
	static const TCHAR POS_digitH  = _T('2');       //* digit-string + Hangul*/

	static const TCHAR POS_VERB  =   _T('V');       //* verb                  */
	static const TCHAR POS_ADJ   =   _T('J');       //* adjective             */
	static const TCHAR POS_XVERB =   _T('W');       //* auxiliary verb        */
	static const TCHAR POS_XADJ  =   _T('K');       //* NOT USED YET          */

	static const TCHAR POS_ADV   =   _T('B');       //* adverb                */
	static const TCHAR POS_DET   =   _T('D');       //* determiner            */
	static const TCHAR POS_EXCL  =   _T('L');       //* exclamation           */

	static const TCHAR POS_JOSA   =  _T('j');       //* Korean Josa           */
	static const TCHAR POS_COPULA =  _T('c');       //* copula '-Wi-'         */
	static const TCHAR POS_EOMI   =  _T('e');       //* final Ending          */
	static const TCHAR POS_PEOMI  =  _T('f');       //* prefinal Ending       */
	static const TCHAR POS_NEOMI  =  _T('n');       //* nominalizing Eomi     */

	static const TCHAR POS_PREFIX =  _T('p');       //* prefixes              */
	static const TCHAR POS_SFX_N  =  _T('s');       //* noun suffixes: '들/적'*/
	static const TCHAR POS_SFX_V  =  _T('t');       //* verb suffixes: '하/되'*/

	static const TCHAR POS_ETC   =   _T('Z');       //* not decided yet       */

	/* ASCII stem may be classified as follows: NOT USED YET    */
	static const TCHAR POS_ALPHA  =  _T('A');       //* English alphabet      */
	static const TCHAR POS_NUMBER =  _T('#');       //* Arabic numbers        */
	static const TCHAR POS_SMARK  =  _T('R');       //* sentence markers      */

	static const TCHAR POS_NVERBK  = _T('Y');       //* guessed as noun+verb  */

	static const TCHAR POS_SQUOTE  = _T('s');       //* single quotation      */
	static const TCHAR POS_DQUOTE  = _T('d');       //* double quotation      */
	static const TCHAR POS_LPAREN  = _T('l');       //* left parenthesis      */
	static const TCHAR POS_RPAREN  = _T('r');       //* right parenthesis     */
	
	
	/**----------------------  불규칙 변형 유형  ------------------------ */	
	static const TCHAR IRR_TYPE_DI = _T('d');  //* ㄷ 불규칙
	static const TCHAR IRR_TYPE_BI = _T('b'); //* ㅂ 불규칙
	static const TCHAR IRR_TYPE_SI = _T('s'); //* ㅅ 불규칙
	static const TCHAR IRR_TYPE_HI = _T('h'); //* ㅎ 불규칙
	static const TCHAR IRR_TYPE_RO = _T('r'); //* 러 불규칙
	static const TCHAR IRR_TYPE_LO = _T('l'); //* 르 불규칙
	static const TCHAR IRR_TYPE_OU = _T('o'); // * 우 불규칙
	static const TCHAR IRR_TYPE_GU = _T('g'); // *거라 불규칙
	static const TCHAR IRR_TYPE_NU = _T('n'); // * 너라 불규칙
	static const TCHAR IRR_TYPE_YO = _T('y'); // * 여 불규칙
	static const TCHAR IRR_TYPE_LI = _T('L'); // * ㄹ 탈락
	static const TCHAR IRR_TYPE_UO = _T('u'); //으 탈락
	static const TCHAR IRR_TYPE_AH = _T('a'); // 아 탈락
	static const TCHAR IRR_TYPE_AE = _T('e'); // 애 축약
	static const TCHAR IRR_TYPE_WA = _T('w'); // 와 축약
	static const TCHAR IRR_TYPE_EI = _T('e'); // 이 축약
	static const TCHAR IRR_TYPE_OE = _T('O'); // 외 축약
		
	
	/**-----------------------  조사의 변이체 유형  ------------------------
    *
	*JOSA_VAR_WiAb	-- '은/는', '이/가', '을/를', '와/과', '아/야' 구분 정보
	*	'는/가/를/와/야'인 경우에 이 값이 set.
	*JOSA_VAR_Wz_tal	-- '로/으로' 구분 정보 (예) '학교로' --> '학교'+'으로'
	*	'으'가 탈락되어 복원된 경우에 이 값이 set.
	*JOSA_VAR_Wi_tal	-- 조사 '고'와 '이고', '라고'와 '이라고' 구분 정보
	*	'이'가 생략되어 복원한 경우에 이 값이 set.
	*JOSA_VAR_Wg_tal	-- 조사 '에서'의 '에' 생략 정보
	*	'에'가 생략되어 복원한 경우에 이 값이 set. '학교서' --> '학교'+'에서'
	*
	*JOSA_VAR_nameWi	-- 인명 뒤에 조음소 '이' 추가되는 경우
	*	'이'를 분석결과에서 삭제한 경우. '승식'+'(이)가'
	*	<참고> 이 경우는 항상 JOSA_VAR_WiAb 현상을 동반한다.
	*JOSA_VAR_preWi	-- 서술격 조사 '이' 앞에 조사가 오는 경우
	*	(예) '에서/부터/에서부터/대로' + '이' + '다'
	*JOSA_VAR_preWi2	-- 서술격 조사 '이' 앞에 조사 & '에' 탈락된 경우
	*	(예) '학교서다' --> '학교'+'(에)서'+'(이)'+'다'
	*
	*JOSA_VAR_Ag	-- '에게'의 변이체 '게'
	*JOSA_VAR_Bg	-- '에게'의 변이체 '께'
	*JOSA_VAR_hbDtg	-- '에게'의 변이체 '한테'
	*
	*$$$ 현재, '승식이한테'의 경우에 정보가 불충분한 점이 있음.
	*-------------------------  조사의 변이체 유형  ------------------------*/

	//	Values for 'jomi.josa'.
	static const int JOSA_VAR_WiAb		= 1;
	static const int JOSA_VAR_Wz_tal	= 2;
	static const int JOSA_VAR_Wi_tal	= 3;
	static const int JOSA_VAR_Wg_tal	= 4;

	static const int JOSA_VAR_nameWi	= 5;
	static const int JOSA_VAR_preWi		= 6;
	static const int JOSA_VAR_preWi2	= 7;

	//Values for 'jomi.josaAgBg'.
	static const int JOSA_VAR_Ag		= 1;
	static const int JOSA_VAR_Bg		= 2;
	static const int JOSA_VAR_hbDtg		= 3;

	/**---------------------  어말어미의 변이체 유형  ----------------------
 	*
	*EOMI_VAR_Wb	-- '아'
	*EOMI_VAR_Wf	-- '어'
	*EOMI_VAR_Wj	-- '여' : '여/거/너/러/라'
	*EOMI_VAR_Wb_tal	-- '아' 탈락
	*EOMI_VAR_Wf_tal	-- '어' 탈락
	*EOMI_VAR_b	-- 'ㅏ'
	*EOMI_VAR_f	-- 'ㅓ'
	*EOMI_VAR_j	-- 'ㅕ'
	*EOMI_VAR_c	-- 'ㅐ'	---> '해서', '까매서/하얘서' 등 ㅎ-불규칙
	*EOMI_VAR_lc	-- 'ㅙ'	---> '되다'에만 적용
	*EOMI_VAR_If, Ib -- '러' 불규칙인 경우
	*
	*EOMI_VAR_Wz_tal	-- 종성 'ㄴ/ㄹ/ㅁ/ㅂ' 및 초성 'ㄹ/ㅁ'으로 시작되는 어미에서 '으' 탈락
	*	<주의> '나/냐/느'로 시작되는 어미들은 '으' 탈락으로 간주하지 않음
	*EOMI_VAR_Uz_tal	-- '습니다'에서 '스' 탈락
	*
	*EOMI_VAR_xv_Wf	-- 보조용언 앞에 오는 어미가 '아/어'
	*EOMI_VAR_xv_Al	-- 보조용언 앞에 오는 어미가 '고'
	*EOMI_VAR_xv_Ag	-- 보조용언 앞에 오는 어미가 '게'
	*
	*EOMI_VAR_Wi_tal	-- 무종성 용언 뒤에서 서술격 조사 '이' 생략
	*
	*$$$ '아/어'에 대한 변이체는 보조용언 앞에 오는 '아/어'에도 적용됨.
	*
	*-----------------------  어말어미의 변이체 유형  ----------------------*/

	//Values for 'jomi.eomi' or 'jomi.xomi'.
	static const int EOMI_VAR_Wb		= 1;
	static const int EOMI_VAR_Wf		= 2;
	static const int EOMI_VAR_Wj		= 3;
	static const int EOMI_VAR_Wb_tal	= 4;
	static const int EOMI_VAR_Wf_tal	= 5;
	static const int EOMI_VAR_b			= 6;
	static const int EOMI_VAR_f			= 7;
	static const int EOMI_VAR_j			= 8;
	static const int EOMI_VAR_c			= 9;
	static const int EOMI_VAR_lc		= 10;
	static const int EOMI_VAR_If		= 11;
	static const int EOMI_VAR_Ib		= 12;

	static const int EOMI_VAR_Wz_tal	= 13;
	static const int EOMI_VAR_Uz_tal	= 14;

	static const int EOMI_VAR_Wi_tal	= 15;

	//Values for 'jomi.xomitype'.
	static const int EOMI_VAR_xv_Wf	= 0; // 아/어
	static const int EOMI_VAR_xv_Al	= 1; // 고
	static const int EOMI_VAR_xv_Ag	= 2;  // 게
	static const int EOMI_VAR_xv_Xi	= 11; // 지

/**---------------------  선어말어미의 변이체 유형  ----------------------
*
*	선어말어미의 변이체 유형 --- 두 가지 정보를 표현
*
*		1. '시' 앞에 조음소 '으'가 오는 경우
*		2. '았/었'의 변이체 정보
*
*	POMI_VAR_WbV	-- '았'
*	POMI_VAR_WfV	-- '었'
*	POMI_VAR_WjV	-- '였'
*	POMI_VAR_V	-- 'ㅆ'
*	POMI_VAR_bV	-- 'ㅏㅆ'
*	POMI_VAR_fV	-- 'ㅓㅆ'
*	POMI_VAR_jV	-- 'ㅕㅆ'
*	POMI_VAR_cV	-- 'ㅐㅆ'	---> '했다'에만 적용됨
*	POMI_VAR_lcV	-- 'ㅙㅆ'	---> '됐다'에만 적용됨
*	POMI_VAR_IfV	-- '렀' ---> '러' 불규칙인 경우
*	POMI_VAR_WzUi	-- '으시' & '었'
*	POMI_VAR_WzUjV	-- '으시' & 'ㅕㅆ', 즉 '으셨'
*
*-----------------------  선어말어미의 변이체 유형  ----------------------*/

	//	Values for 'jomi.pomi'.
	static const int POMI_VAR_WbV	= 1;
	static const int POMI_VAR_WfV	= 2;
	static const int POMI_VAR_WjV	= 3;
	static const int POMI_VAR_V		= 4;
	static const int POMI_VAR_bV	= 5;
	static const int POMI_VAR_fV	= 6;
	static const int POMI_VAR_jV	= 7;
	static const int POMI_VAR_cV	= 8;
	static const int POMI_VAR_lcV	= 9;
	static const int POMI_VAR_IfV	= 10;
	static const int POMI_VAR_WzUi	= 11;
	static const int POMI_VAR_WzUjV	= 12;

/**---------------------  조사/어미 외 기타 정보 ----------------------
*
*	RMA_RESULT --- the result is got from 'hangul.rma'
*	GUESS_ABBR --- verb stem is guessed as abbr. 'ki/kg/Zi/...'
*
*	GUESS_CNOUN -- stem is guessed as noun + noun + ...
*	GUESS_PNOUN	-- proper noun with Jongsong: articulative 'Wi' dropped.
*
*	GUESS_NPREF -- noun stem is guessed as prefix 'Gc/Ul' + noun
*	GUESS_VPREF -- verb stem is guessed as prefix 'WbD/QlU' + verb
*	GUESS_NVERB -- verb stem is guessed as noun + verb + ...
*
*-----------------------  조사/어미 외 기타 정보 ----------------------*/

	//	Values for 'jomi.zzz'.
	static const int RMA_RESULT		= 1;
	static const int GUESS_ABBR		= 2;

	static const int GUESS_CNOUN	= 3;
	static const int GUESS_PNOUN	= 4;

	static const int GUESS_NPREF	= 5;
	static const int GUESS_VPREF	= 6;
	static const int GUESS_NVERB	= 7;
};

#endif
