/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_morph_WordEntry
#define _lucene_analysis_kr_morph_WordEntry

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/StdHeader.h"
#include "../utils/String.h"
#include "CompoundEntry.h"

CL_NS_DEF2(analysis, kr)

class WordEntry {
public:

public:
	static const int IDX_NOUN   = 0;
	static const int IDX_VERB   = 1;
	static const int IDX_BUSA   = 2;
	static const int IDX_DOV    = 3;
	static const int IDX_BEV    = 4;
	static const int IDX_REGURA = 5;
	static const int IDX_PREFIX = 6;
	static const int IDX_SUFFIX = 7;
	static const int IDX_NCNOUN = 8;  // 마지막에 복합명사로 올 수 없는 경
	static const int IDX_CNOUNX = 9;  // 중간에 복합명사로 올 수 없는 경우

private:
	/**
	 * 단어
	 */
	String word;
	
	/**
	 * 단어특성
	 */
	String features; // FIXME
	
	CompoundEntryList compounds;

public:
	WordEntry() {		
	}
	
	WordEntry(const String& _word) : word(_word) {}
	
	WordEntry(const String& _word, const String& cs) :
		word(_word), features(cs) {}

	WordEntry(const String& _word, const CompoundEntryList& c) :
		word(_word), compounds(c) {}

	WordEntry(const String& _word, const String& cs, const CompoundEntryList& c) :
		word(_word), features(cs), compounds(c) {}
	
	void setWord(const String& w) {
		this->word = w;
	}
	
	const String& getWord() const {
		return this->word;
	}
	
	void setFeatures(const String& cs) {
		this->features = cs;
	}
	
	TCHAR getFeature(int index) {
		if (/*features == NULL ||*/ features.length() < index)
			return '0';		
		return features[index];
	}
	
	String& getFeatures() {
		return this->features;
	}
	
	void setCompounds(CompoundEntryList &c) {
		this->compounds = c;
	}
	
	const CompoundEntryList& getCompounds() const {
		return this->compounds;
	}	
};

typedef CL_NS(util)::CLVector<WordEntry,
						CL_NS(util)::Deletor::NullVal<WordEntry> > WordEntryList;
								   
CL_NS_END2
#endif
