/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_utils_DictionaryUtil
#define _lucene_analysis_kr_utils_DictionaryUtil

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include <ext/hash_map>
#include "CLucene/StdHeader.h"
#include "../Trie.h"
#include "../morph/WordEntry.h"
#include "../morph/MorphException.h"
#include "FileUtil.h"

CL_NS_USE(util)

CL_NS_DEF2(analysis, kr)

class DictionaryUtil {
private:
	class Dictionary;
	static Dictionary *dictionary;
	
public:
	typedef Trie<String, WordEntry>::ValueIterator EntryIterator;
	typedef Trie<String, WordEntry>::Position EntryPosition;	
	
public:		
	static EntryIterator findWithPrefix(const String& prefix) throw(MorphException);

	static EntryPosition findWord(const String& key) throw(MorphException);
	
	static EntryPosition findNoun(const String& key) throw(MorphException);
	
	static EntryPosition findCNoun(const String& key) throw(MorphException);
	
	static EntryPosition findVerb(const String& key) throw(MorphException);
	
	static EntryPosition findAdverb(const String& key) throw(MorphException);
	
	static EntryPosition findIrrVerb(const String& key, TCHAR irrType) throw(MorphException);
	
	static EntryPosition findBeVerb(const String& key) throw(MorphException);
	
	static EntryPosition findDoVerb(const String& key) throw(MorphException);
	
	static bool existJosa(const String& str) throw(MorphException);
	
	static bool existEomi(const String& str)  throw(MorphException);
	
	static bool existPrefix(const String& str)  throw(MorphException);
	
	static bool existSuffix(const String& str)  throw(MorphException);
	
	/**
	 * ㄴ,ㄹ,ㅁ,ㅂ과 eomi 가 결합하여 어미가 될 수 있는지 점검한다.
	 * @param s
	 * @param end
	 * @return
	 */
	static String combineAndEomiCheck(TCHAR s, const String& eomi) throw(MorphException);
	
private:
	/**
	 * 사전을 로드한다.
	 */
	static void loadDictionary() throw(MorphException);
	
	static void compoundArrayToList(const String& source, const StringList& arr,
									/*out*/ CompoundEntryList& list);

private:
	class Dictionary {
		friend class DictionaryUtil;

	private:
		struct StringHash {
			// FIXME: overflow? (wchar_t may be 4bytes)
			size_t operator()(const String& _Key) const	{
				unsigned long __h = 0;
				for (int i = 0 ; i < _Key.length(); i++)
					__h = 5 * __h + _Key[i];
				return size_t(__h);
			}

		};

		struct StringEqual {
			bool operator()(const String& s1, const String& s2) const {
				return (s1.compare(s2) == 0) ? true : false;
			}
		};
		typedef __gnu_cxx::hash_map<String, String, StringHash, StringEqual> HashMap;
		
		Trie<String,WordEntry> trie;
		HashMap josas;
		HashMap eomis; 
		HashMap prefixs;	 
		HashMap suffixs;
		
	public:
		Dictionary();		
	};
};

CL_NS_END2
#endif
