/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "EomiUtil.h"
#include "SyllableUtil.h"
#include "DictionaryUtil.h"
#include "MorphUtil.h"

CL_NS_DEF2(analysis, kr)

const TCHAR* EomiUtil::RESULT_FAIL		= _T("0");	
const TCHAR* EomiUtil::RESULT_SUCCESS	= _T("1");
const TCHAR* EomiUtil::verbSuffix[] = {
	_T("이"),_T("하"),_T("되"),_T("스럽"),_T("스러우"),_T("시키"),_T("있"),_T("없"),_T("같"),_T("당하"),_T("만하"),_T("드리"),_T("받"), NULL
};


/**
 * 가장 길이가 긴 어미를 분리한다.
 * @param term
 * @return
 * @throws MorphException
 */
void EomiUtil::longestEomi(const String& term, String result[]) throw(MorphException) {
	result[0] = term;
		
	String stem;
	String eomi;
		
	for(int i = term.length(); i > 0; i--) {
		stem = term.substring(0,i);			
		TCHAR syllable;
		if (i != term.length()) {
			eomi = term.substring(i);
			syllable = eomi.charAt(0);
		} else {
			syllable = stem.charAt(i-1);
			eomi=_T("");
		}
		const String& efeature = SyllableUtil::getFeature(syllable);
		
		if (SyllableUtil::isAlpanumeric(stem.charAt(i-1))) break;
			
		TCHAR jasos[3] = {0, };
		int jasos_length = MorphUtil::decompose(stem.charAt(i-1), jasos);
	
		if(!eomi.empty() && !DictionaryUtil::existEomi(eomi)) {
			// do not anything.
		}
		else if (jasos_length > 2 &&
				 (jasos[2]==_T('ㄴ')||jasos[2]==_T('ㄹ')||jasos[2]==_T('ㅁ')||jasos[2]==_T('ㅂ')) &&
				 DictionaryUtil::combineAndEomiCheck(jasos[2], eomi) != _T(""))
		{
			result[0] = String(1, MorphUtil::makeChar(stem.charAt(i-1), 0));
			if(i!=0) result[0] = stem.substring(0,i-1).append(result[0]);
			result[1] = String(1, jasos[2]);
		}
		else if (i > 0 &&
				 (stem.endsWith(_T("하")) && eomi.equals(_T("여"))) ||
				 (stem.endsWith(_T("가")) && eomi.equals(_T("거라"))) ||
				 (stem.endsWith(_T("오")) && eomi.equals(_T("너라"))) )
		{
			result[0] = stem;
			result[1] = eomi;			
		}
		else if (jasos_length == 2 &&
				 (!stem.endsWith(_T("아")) && !stem.endsWith(_T("어"))) &&
				 (jasos[1]==_T('ㅏ')||jasos[1]==_T('ㅓ')||jasos[1]==_T('ㅔ')||jasos[1]==_T('ㅐ')) &&
				 (DictionaryUtil::combineAndEomiCheck(_T('어'), eomi) != _T("")) )
		{		
			// CHECK: char[] chs = MorphUtil.decompose(stem.charAt(stem.length()-1));				
			result[0] = stem;
			result[1] = String(_T("어")).append(eomi);
		}
		else if ((jasos[1]==_T('ㅘ')||jasos[1]==_T('ㅝ')||jasos[1]==_T('ㅕ')||jasos[1]==_T('ㅐ')||jasos[1]==_T('ㅒ')) &&
				 (DictionaryUtil::combineAndEomiCheck(_T('어'), eomi) != _T("")) )
		{
			String end;
			if(jasos[1]==_T('ㅘ'))
				end = MorphUtil::makeChar(stem.charAt(i-1), 8, 0) + _T("아");	
			else if(jasos[1]==_T('ㅝ'))
				end = MorphUtil::makeChar(stem.charAt(i-1), 13, 0) + _T("어");	
			else if(jasos[1]==_T('ㅕ'))
				end = MorphUtil::makeChar(stem.charAt(i-1), 6, 0);
			else if(jasos[1]==_T('ㅐ'))
				end = MorphUtil::makeChar(stem.charAt(i-1), 0, 0) + _T("어");	
			else if(jasos[1]==_T('ㅒ'))
				end = MorphUtil::makeChar(stem.charAt(i-1), 20, 0) + _T("애");										
				
			if(jasos_length==3) {					
				end = end.substring(0,end.length()-1)
						.append(1, MorphUtil::replaceJongsung(end.charAt(end.length()-1),stem.charAt(i-1)) );
			}
				
			if(stem.length()<2) result[0] = end;
			else result[0] = stem.substring(0,stem.length()-1).append(end);
			result[1] = eomi;	
				
		}
		else if(!efeature.empty() && efeature[SyllableUtil::IDX_EOMI1] != _T('0') &&
				DictionaryUtil::existEomi(eomi))
		{
			if(! (((jasos_length==2&&jasos[0]==_T('ㄹ')) || (jasos_length==3&&jasos[2]==_T('ㄹ'))) &&
				 eomi.equals(_T("러"))) ) { // ㄹ 불규칙은 예외
				result[0] = stem;
				result[1] = eomi;
			}
		}

		if (!efeature.empty() && efeature[SyllableUtil::IDX_EOMI2]==_T('0')) break;
	}		
}	

	
/**
 * 선어말어미를 분석한다.
 * @param stem
 * @return
 */
void EomiUtil::splitPomi(const String& _stem, String results[]) throw(MorphException)  {
	
	// results[0]:성공(1)/실패(0), results[1]: 어근, results[2]: 선어말어미
	results[0] = _stem;

	if(_stem.length()==0 || _stem.equals(_T("있")) ) return;
	
	String stem = _stem;
	const String& chrs = _stem;
	int len = chrs.length();
	String pomi;
	int index = len-1;

	TCHAR jaso[3] = { 0. };
	int jaso_length = MorphUtil::decompose(chrs[index], jaso);
		
	if (chrs[index]!=_T('시')&&chrs[index]!=_T('ㅆ')&&jaso[jaso_length-1]!=_T('ㅆ') )
		return;  // 선어말어미가 발견되지 않았다
		
	if(chrs[index]==_T('겠')) {
		pomi = _T("겠");
		setPomiResult(results, stem.substring(0,index), pomi);		
		if(--index<=0||
		   (chrs[index]!=_T('시')&&chrs[index]!=_T('ㅆ')&&jaso[jaso_length-1]!=_T('ㅆ'))) 
			return; // 다음이거나 선어말어미가 없다면...
		jaso_length = MorphUtil::decompose(chrs[index], jaso);
	}

	if(chrs[index]==_T('었')) { // 시었, ㅆ었, 었
		pomi = chrs[index]+pomi;	
		setPomiResult(results,stem.substring(0,index),pomi);		
		if(--index<=0||
		   (chrs[index]!=_T('시')&&chrs[index]!=_T('ㅆ')&&jaso[jaso_length-1]!=_T('ㅆ'))) 
			return; // 다음이거나 선어말어미가 없다면...				
		jaso_length = MorphUtil::decompose(chrs[index], jaso);
	}

	if(chrs[index]==_T('였')){
		pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;	
		if(index>0&&chrs[index-1]==_T('하')) 
			stem = stem.substring(0,index);	
		else
			stem = stem.substring(0,index)+_T("이");
		setPomiResult(results,stem,pomi);	
	}else if(chrs[index]==_T('셨')){
		pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;	
		stem = stem.substring(0,index)+_T("시");		
		setPomiResult(results,stem,pomi);				
	}else if(chrs[index]==_T('았')||chrs[index]==_T('었')) {
		pomi = chrs[index]+pomi;	
		setPomiResult(results,stem.substring(0,index),pomi);		
		if(--index<=0||
		   (chrs[index]!=_T('시')&&chrs[index]!=_T('으'))) return; // 다음이거나 선어말어미가 없다면...				
		jaso_length = MorphUtil::decompose(chrs[index], jaso);		
	}else if(jaso_length==3&&jaso[2]==_T('ㅆ')) {
		
		if(jaso[0]==_T('ㅎ')&&jaso[1]==_T('ㅐ')) {			 
			pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;	
			stem = stem.substring(0,index)+_T("하");	
		}else if(jaso[0]!=_T('ㅇ')&&(jaso[1]==_T('ㅏ')||jaso[1]==_T('ㅓ')||jaso[1]==_T('ㅔ')||jaso[1]==_T('ㅐ'))) {		
			pomi = _T("었")+pomi;
			stem = stem.substring(0,index)+MorphUtil::makeChar(chrs[index], 0);				
		}else if(jaso[0]!=_T('ㅇ')&&(jaso[1]==_T('ㅙ'))) {
			pomi = _T("었")+pomi;
			stem = stem.substring(0,index)+MorphUtil::makeChar(chrs[index],11, 0);				
		} else if(jaso[1]==_T('ㅘ')) {			
			pomi = MorphUtil::replaceJongsung(_T('아'),chrs[index])+pomi;	
			stem = stem.substring(0,index)+_T("오");
		} else if(jaso[1]==_T('ㅝ')) {
			pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;	
			stem = stem.substring(0,index)+_T("우");
		} else if(jaso[1]==_T('ㅕ')) {			
			pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;
			stem = stem.substring(0,index)+MorphUtil::makeChar(chrs[index],20, 0);				
		} else if(jaso[1]==_T('ㅐ')) {
			pomi = MorphUtil::replaceJongsung(_T('어'),chrs[index])+pomi;
			stem = stem.substring(0,index);
		} else if(jaso[1]==_T('ㅒ')) {
			pomi = MorphUtil::replaceJongsung(_T('애'),chrs[index])+pomi;	
			stem = stem.substring(0,index);
		} else {
			pomi = _T("었")+pomi;
		}

		setPomiResult(results,stem,pomi);				
		if(--index<=0||
		   (chrs[index]!=_T('시')&&chrs[index]!=_T('으'))) return; // 다음이거나 선어말어미가 없다면...				
		jaso_length = MorphUtil::decompose(chrs[index], jaso);				
	}
		
	TCHAR nChrs[3] = {0, };
	int nChrs_length = 0;
	if(index>0) nChrs_length = MorphUtil::decompose(chrs[index-1], nChrs);

	if(nChrs_length==2&&chrs[index]==_T('시')&&(chrs.length()<=index+1||
												(chrs.length()>index+1&&chrs[index+1]!=_T('셨')))) {
		if (!DictionaryUtil::findWord(results[0]).isNull()) return;  //'시'가 포함된 단어가 있다. 성가시다/도시다/들쑤시다 
		pomi = chrs[index]+pomi;	
		setPomiResult(results,stem.substring(0,index),pomi);			
		if(--index==0||chrs[index]!=_T('으')) return; // 다음이거나 선어말어미가 없다면...				
		jaso_length = MorphUtil::decompose(chrs[index], jaso);
	}

	if(index>0) nChrs_length = MorphUtil::decompose(chrs[index-1], nChrs);
	if(chrs.length()>index+1&&nChrs_length==3&&(chrs[index+1]==_T('셨')||chrs[index+1]==_T('시'))&&chrs[index]==_T('으')) {
		pomi = chrs[index]+pomi;	
		setPomiResult(results,stem.substring(0,index),pomi);		
	}
	
	return;
}


/**
 * 불규칙 용언의 원형을 구한다.
 * @param output
 * @return
 * @throws MorphException
 */
void EomiUtil::irregular(AnalysisOutput& output, /*out*/ AnalysisOutputList& results) {	
	if(output.getStem().length()==0) 
		return;		
		
	const String& ending = (output.getPomi().empty()) ? output.getEomi() :
														output.getPomi();
		
	StringListList irrs;
		
	irregularStem(irrs, output.getStem(), ending);
	irregularEnding(irrs, output.getStem(), ending);
	irregularAO(irrs, output.getStem(), ending);

	StringListList::iterator iter;
	for (iter = irrs.begin(); iter != irrs.end(); iter++) {
		results.push_back(AnalysisOutput(output));
		AnalysisOutput &result = results.back();
		result.setStem((*iter)[0]);
		if(output.getPatn()==PatternConstants::PTN_VM) {
			if(output.getPomi().empty()) result.setEomi((*iter)[1]);
			else result.setPomi((*iter)[1]);
		}	
	}				
				
	return;		
}

/**
 * 어간만 변하는 경우
 * @param results
 * @param stem
 * @param ending
 */
void EomiUtil::irregularStem(StringListList& results, const String& stem, const String& ending) {	

	TCHAR feCh = ending.charAt(0);
	TCHAR fechJaso[3];
	int fechJaso_length = MorphUtil::decompose(feCh, fechJaso);
	
	TCHAR ls = stem.charAt(stem.length()-1);
	TCHAR lsJaso[3];
	int lsJaso_length = MorphUtil::decompose(ls, lsJaso);
	
	if(feCh==_T('아')||feCh==_T('어')||feCh==_T('으')) {
		if(lsJaso[lsJaso_length-1]==_T('ㄹ')) { // ㄷ 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-1)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),7)) );
			irr.push_back(ending);
			irr.push_back(PatternConstants::IRR_TYPE_DI);
		} else if(lsJaso_length==2) { // ㅅ 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-1)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),19)) );
			irr.push_back(ending);
			irr.push_back(PatternConstants::IRR_TYPE_SI);
		}			
	}
		
	if((fechJaso[0]==_T('ㄴ')||fechJaso[0]==_T('ㄹ')||fechJaso[0]==_T('ㅁ')||	feCh==_T('오')||feCh==_T('시'))
	   &&(ls==_T('우'))) { // ㅂ 불규칙
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),17)) );
		irr.push_back(ending);
		irr.push_back(PatternConstants::IRR_TYPE_BI);
	}
		
	if((fechJaso[0]==_T('ㄴ')||fechJaso[0]==_T('ㅂ')||fechJaso[0]==_T('ㅅ')||	feCh==_T('오'))
	   &&(lsJaso_length==2)) { // ㄹ 탈락
 		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),8)) );
		irr.push_back(ending);
		irr.push_back(PatternConstants::IRR_TYPE_LI);
	}
		
	if(lsJaso_length==2
	   &&(fechJaso[0]==_T('ㄴ')||fechJaso[0]==_T('ㄹ')||fechJaso[0]==_T('ㅁ')||fechJaso[0]==_T('ㅂ')||
		  lsJaso[1]==_T('ㅏ')||lsJaso[1]==_T('ㅓ')||lsJaso[1]==_T('ㅑ')||lsJaso[1]==_T('ㅕ'))
	   &&!stem.equals(_T("나"))) { // ㅎ 불규칙, 그러나 [낳다]는 ㅎ 불규칙이 아니다.
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),27)) );
		irr.push_back(ending);
		irr.push_back(PatternConstants::IRR_TYPE_HI);
	}		
}

	
/**
 * 어미만 변하는 경우
 * @param results
 * @param stem
 * @param ending
 */
void EomiUtil::irregularEnding(StringListList& results, const String& stem, const String& ending) {
	if(ending.startsWith(_T("ㅆ"))) return;
		
	char feCh = ending.charAt(0);
	char ls = stem.charAt(stem.length()-1);

	if(feCh==_T('러')&&ls==_T('르')) { // '러' 불규칙
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem);
		irr.push_back(_T("어")+ending.substring(1));
		irr.push_back(PatternConstants::IRR_TYPE_RO);
	}
	else if(ending.equals(_T("라")) && stem.equals( _T("가거"))) { // '거라' 불규칙
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1));
		irr.push_back(_T("어라"));
		irr.push_back(PatternConstants::IRR_TYPE_GU);
	}
	else if(ending.equals(_T("라")) && stem.equals(_T("오너"))) { // '너라' 불규칙
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1));
		irr.push_back(_T("어라"));
		irr.push_back(PatternConstants::IRR_TYPE_NU);
	}
		
	if(ending.equals(_T("여")) && ls==_T('하')) { // '여' 불규칙
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem);
		irr.push_back(_T("어"));
		irr.push_back(PatternConstants::IRR_TYPE_NU);
	}
}

/**
 * 어간과 어미가 모두 변하는 경우
 * @param results
 * @param stem
 * @param ending
 */
void EomiUtil::irregularAO(StringListList& results, const String& stem, const String& ending) { // 
		
	TCHAR ls = stem.charAt(stem.length()-1);
	TCHAR lsJaso[3];
	int lsJaso_length = MorphUtil::decompose(ls, lsJaso);
		
	if(lsJaso_length<2) return;
		
	if(lsJaso[1]==_T('ㅘ')) {
		if(stem.endsWith(_T("도와")) || stem.endsWith(_T("고와"))) { // '곱다', '돕다'의 'ㅂ' 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-2)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-2),17)) ); // + 'ㅂ'
			irr.push_back(makeTesnseEomi(_T("아"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_BI);
		}
		else { // _T('와') 축약
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-1)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),8,0)) ); // 자음 + ㅗ 
			irr.push_back(makeTesnseEomi(_T("아"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_WA);
		}
	}
	else if(stem.endsWith(_T("퍼"))) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),18,0)) ); // 자음 + -
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_WA);
	}
	else if(lsJaso[1]==_T('ㅝ')) {
		if(stem.length()>=2) { // 'ㅂ' 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-2)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-2),17)) ); // + 'ㅂ'
			irr.push_back(makeTesnseEomi(_T("어"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_BI);
		}
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),13,0)) ); // 자음 + ㅗ 
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_WA);					 
	}
	else if(stem.length()>=2&&ls==_T('라')) {
		TCHAR ns[3]; 
		int ns_length = MorphUtil::decompose(stem.charAt(stem.length()-2), ns);

		if(ns_length==3&&ns[2]==_T('ㄹ')) { // 르 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-2)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-2),0))
						  .append(_T("르")) );
			irr.push_back(makeTesnseEomi(_T("아"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_RO);
		}			
	}
	else if(stem.length()>=2&&ls==_T('러')) {
		TCHAR ns[3]; 
		int ns_length =	MorphUtil::decompose(stem.charAt(stem.length()-2), ns);
		
		if(stem.charAt(stem.length()-2)==_T('르')) { // 러 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-1));
			irr.push_back(makeTesnseEomi(_T("어"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_LO);
		}
		else if(ns_length==3&&ns[2]==_T('ㄹ')) { // 르 불규칙
			results.push_back(StringList());
			StringList &irr = results.back();
			irr.push_back(stem.substring(0,stem.length()-2)
						  .append(MorphUtil::makeChar(stem.charAt(stem.length()-2),0))
						  .append(_T("르")) );
			irr.push_back(makeTesnseEomi(_T("어"),ending));
			irr.push_back(PatternConstants::IRR_TYPE_RO);
		}
	}
	else if(stem.endsWith(_T("펴")) || stem.endsWith(_T("켜"))) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),20,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_EI);
	}
	else if(stem.endsWith(_T("해"))) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),0,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_EI);
	}
	else if(lsJaso_length==2&&lsJaso[1]==_T('ㅏ')) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),18,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_UO);
	}
	else if(lsJaso_length==2&&lsJaso[1]==_T('ㅓ')) {
		// 으 탈락
		{
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),18,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_UO);
		}

		//	 아 불규칙
		{
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem);
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_AH);
		}
	}
	else if(lsJaso[1]==_T('ㅕ')) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),20,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_EI);
	}
	else if(lsJaso[1]==_T('ㅙ')) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),11,0)) );
		irr.push_back(makeTesnseEomi(_T("어"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_OE);
	}
	else if(lsJaso[1]==_T('ㅐ')) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),0,27)) );
		irr.push_back(makeTesnseEomi(_T("아"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_HI);
	}
	else if(lsJaso[1]==_T('ㅒ')) {
		results.push_back(StringList());
		StringList &irr = results.back();
		irr.push_back(stem.substring(0,stem.length()-1)
					  .append(MorphUtil::makeChar(stem.charAt(stem.length()-1),2,27)) );
		irr.push_back(makeTesnseEomi(_T("아"),ending));
		irr.push_back(PatternConstants::IRR_TYPE_HI);
	}
		
}

/**
 * 시제 선어미말을 만들어서 반환한다.
 * @param preword  '아' 또는 '어'
 * @param endword  어미[선어미말을 포함]
 * @return '았' 또는 '었'을 만들어서 반환한다.
 */
String EomiUtil::makeTesnseEomi(const String& preword, const String& endword) {

	if(preword.length()==0) return endword;
	if(endword.length()==0) return preword;

	if(endword.charAt(0)==_T('ㅆ')) {
		return preword.substring(0,preword.length()-1)+
			MorphUtil::makeChar(preword.charAt(preword.length()-1),20)+endword.substring(1,endword.length());		
	} else if(endword.charAt(0)==_T('ㄴ')) {
		return preword.substring(0,preword.length()-1)+
			MorphUtil::makeChar(preword.charAt(preword.length()-1),4)+endword.substring(1,endword.length());
	} else if(endword.charAt(0)==_T('ㄹ')) {
		return preword.substring(0,preword.length()-1)+
			MorphUtil::makeChar(preword.charAt(preword.length()-1),8)+endword.substring(1,endword.length());	
	} else if(endword.charAt(0)==_T('ㅁ')) {
		return preword.substring(0,preword.length()-1)+
			MorphUtil::makeChar(preword.charAt(preword.length()-1),16)+endword.substring(1,endword.length());					
	} else if(endword.charAt(0)==_T('ㅂ')) {
		return preword.substring(0,preword.length()-1)+
			MorphUtil::makeChar(preword.charAt(preword.length()-1),17)+endword.substring(1,endword.length());
	}
	return preword+endword;		
}

/**
 * '음/기' + '이' + 어미, '에서/부터/에서부터' + '이' + 어미 인지 조사한다.
 * @param stem
 * @return
 */
bool EomiUtil::endsWithEEomi(const String& stem) {
	int len = stem.length();
	if(len<2||!stem.endsWith(_T("이"))) return false;
	  
	TCHAR jasos[3];
	int jasos_length = MorphUtil::decompose(stem.charAt(len-2), jasos);
	if(jasos_length==3&&jasos[2]==_T('ㅁ'))
		return true;
	else {
		int index = stem.rfind(_T("기"));
		if(index==-1) index = stem.rfind(_T("에서"));
		if(index==-1) index = stem.rfind(_T("부터"));
		if(index==-1) return false;
		return true;
	}
}


/**
 * 어미를 분리한다.
 * 
 * 1. 규칙용언과 어간만 바뀌는 불규칙 용언
 * 2. 어미가 종성 'ㄴ/ㄹ/ㅁ/ㅂ'으로 시작되는 어절
 * 3. '여/거라/너라'의 불규칙 어절
 * 4. 어미 '아/어'가 탈락되는 어절
 * 5. '아/어'의 변이체 분리
 * 
 * @param stem
 * @param end
 * @return
 * @throws MorphException
 */
// FIXME: return bool;
void EomiUtil::splitEomi(const String& stem, const String& _end,
						 /*out*/ String strs[]) throw(MorphException)
{
	String end = _end;
	
	int strlen = stem.length();
	if(strlen==0) return;
	
	TCHAR estem = stem.charAt(strlen-1);
	TCHAR chrs[3];
	int chrs_length = MorphUtil::decompose(estem, chrs);
	if(chrs_length==1) return; // 한글이 아니라면...

	if((chrs_length==3)&&(chrs[2]==_T('ㄴ')||chrs[2]==_T('ㄹ')||chrs[2]==_T('ㅁ')||chrs[2]==_T('ㅂ'))&&
	   EomiUtil::IsNLMBSyl(estem,chrs[2])&&
	   !DictionaryUtil::combineAndEomiCheck(chrs[2], end).empty())
	{
		strs[1] = chrs[2];
		if(end.length()>0) strs[1] += end;
		strs[0] = stem.substring(0,strlen-1) + MorphUtil::makeChar(estem, 0);	
	}
	else if(estem==_T('해')&&DictionaryUtil::existEomi(_T("어")+end)) {			
		strs[0] = stem.substring(0,strlen-1)+_T("하");
		strs[1] = _T("어")+end;	
	}
	else if(estem==_T('히')&&DictionaryUtil::existEomi(_T("이")+end)) {			
		strs[0] = stem.substring(0,strlen-1)+_T("하");
		strs[1] = _T("이")+end;				
	}
	else if(chrs[0]!=_T('ㅇ')&&
			(chrs[1]==_T('ㅏ')||chrs[1]==_T('ㅓ')||chrs[1]==_T('ㅔ')||chrs[1]==_T('ㅐ'))&&
			(chrs_length==2 || SyllableUtil::getFeature(estem)[SyllableUtil::IDX_YNPAH]==_T('1'))&&
			(!DictionaryUtil::combineAndEomiCheck(_T('어'), end).empty()))
	{		
		strs[0] = stem;
		if(chrs_length==2) strs[1] = _T("어")+end;	
		else strs[1] = end;	
	}
	else if(stem.endsWith(_T("하")) && end.equals(_T("여"))) {			
		strs[0] = stem;
		strs[1] = _T("어");	
	}
	else if((chrs_length==2)&&(chrs[1]==_T('ㅘ')||chrs[1]==_T('ㅙ')||chrs[1]==_T('ㅝ')||chrs[1]==_T('ㅕ')||chrs[1]==_T('ㅐ')||chrs[1]==_T('ㅒ'))&&
			(!DictionaryUtil::combineAndEomiCheck(_T('어'), end).empty()) )
	{
		String sb;
			
		if(strlen>1) sb.append(stem.substring(0,strlen-1));
			
		if(chrs[1]==_T('ㅘ'))
			sb.append(MorphUtil::makeChar(estem, 8, 0)).append(MorphUtil::replaceJongsung(_T('아'),estem));	
		else if(chrs[1]==_T('ㅝ'))
			sb.append(MorphUtil::makeChar(estem, 13, 0)).append(MorphUtil::replaceJongsung(_T('어'),estem));	
		else if(chrs[1]==_T('ㅙ'))
			sb.append(MorphUtil::makeChar(estem, 11, 0)).append(MorphUtil::replaceJongsung(_T('어'),estem));				
		else if(chrs[1]==_T('ㅕ'))
			sb.append(MorphUtil::makeChar(estem, 20, 0)).append(MorphUtil::replaceJongsung(_T('어'),estem));
		else if(chrs[1]==_T('ㅐ'))
			sb.append(MorphUtil::makeChar(estem, 0, 0)).append(MorphUtil::replaceJongsung(_T('어'),estem));
		else if(chrs[1]==_T('ㅒ'))
			sb.append(MorphUtil::makeChar(estem, 20, 0)).append(MorphUtil::replaceJongsung(_T('애'),estem));				
		
		strs[0] = sb;
		
		end = strs[0].substring(strs[0].length()-1)+end;				
		strs[0] = strs[0].substring(0,strs[0].length()-1);
			
		strs[1] = end;		
	}
	else if(!end.empty()&&DictionaryUtil::existEomi(end)) {		
		strs[0] = stem;
		strs[1] = end;
	}
}

CL_NS_END2
