/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "IrregularUtil.h"
#include "DictionaryUtil.h"
#include "MorphUtil.h"
#include "../morph/WordEntry.h"

CL_NS_DEF2(analysis, kr)

bool IrregularUtil::restoreIrregularVerb(const String& start, const String& end,
												/*out*/ String results[]) throw(MorphException) {
	TCHAR jasos[3];
	int jasos_length = 0;

	if(end.length()>0) jasos_length = MorphUtil::decompose(end.charAt(0), jasos);

	if(end.startsWith(_T("ㄴ"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreHIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreELIrregular(start,end,results);
		if(isRestored) return true;				
	}else if(end.startsWith(_T("ㄹ"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreHIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreELIrregular(start,end,results);
		if(isRestored) return true;			
	}else if(end.startsWith(_T("ㅁ"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;
		isRestored = restoreHIrregular(start,end,results);
		if(isRestored) return true;				
	}else if(end.startsWith(_T("ㅂ"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreHIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreELIrregular(start,end,results);
		if(isRestored) return true;					
	}else if(start.endsWith(_T("우"))||start.endsWith(_T("오"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;				
	}else if(end.startsWith(_T("오"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;	
	}else if(end.startsWith(_T("시"))) {
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreELIrregular(start,end,results);
		if(isRestored) return true;				
	}else if(end.startsWith(_T("으"))) {			
		bool isRestored = restoreBIrregular(start,end,results);
		if(isRestored) return true;				
	}else if(jasos_length>1&&jasos[0]==_T('ㅇ')&&(jasos[1]==_T('ㅓ')||jasos[1]==_T('ㅏ'))) {			
		bool isRestored = restoreDIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreSIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreLIrregular(start,end,results);
		if(isRestored) return true;		
		isRestored = restoreHIrregular(start,end,results);
		if(isRestored) return true;	
		isRestored = restoreUIrregular(start,end,results);
		if(isRestored) return true;		
		isRestored = restoreRUIrregular(start,end,results);
		if(isRestored) return true;						
	}else if(jasos_length>1&&jasos[0]==_T('ㅇ')&&jasos[1]==_T('ㅡ')) {			
		bool isRestored = restoreDIrregular(start,end,results);
		if(isRestored) return true;		
		isRestored = restoreSIrregular(start,end,results);
		if(isRestored) return true;				
	}else if((start.equals(_T("가")) && end.equals(_T("거라")))||
			 (start.equals(_T("오")) && end.equals(_T("너라"))) ) {
		results[0] = start;
		results[1] = end;
		return true;
	}
		
	return false;
		
}


/**
 * ㅂ 불규칙 원형을 복원한다. (돕다, 곱다)
 * @param start
 * @param end
 * @return
 * @throws MorphException
 */
bool IrregularUtil::restoreBIrregular(const String& _start, const String& end,
											 /*out*/ String results[]) throw(MorphException) {

	String start = _start;
	
	if(start.empty()) return false; // CHECK: end.empty() ?	    
	if(start.length()<2) return false;	    
	if(!(start.endsWith(_T("오"))||start.endsWith(_T("우")))) return false;
	    
	TCHAR convEnd = MorphUtil::makeChar(end.charAt(0), 0);
	if(end.equals(_T("ㅁ"))||end.equals(_T("ㄴ"))||end.equals(_T("ㄹ"))||
	   convEnd==_T('아')||convEnd==_T('어')) { // 도우(돕), 고오(곱), 스러우(스럽) 등으로 변형되므로 반드시 2자 이상임
			
		TCHAR ch = start.charAt(start.length()-2);
		ch = MorphUtil::makeChar(ch, 17);
		
		if(start.length()>2)
			start = start.substring(0,start.length()-2).append(ch);
		else
			start = ch;

		DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
		if(!entry.isNull() && entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_BIUP) {
			results[0] = start;
			results[1] = end;
			return true;
		}			
	}			

	return false; 		
}
	
/**
 * ㄷ 불규칙 원형을 복원한다. (깨닫다, 묻다)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreDIrregular(const String& _start, const String& end,
									  /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(start.equals(_T(""))) return false;	
	
	TCHAR ch = start.charAt(start.length()-1);
	TCHAR jasos[3];
	int jasos_length = MorphUtil::decompose(ch, jasos);
	if(jasos_length!=3||jasos[2]!=_T('ㄹ')) return false;
		
	ch = MorphUtil::makeChar(ch, 7);
	if(start.length()>1) 
		start = start.substring(0,start.length()-1).append(ch);
	else
		start = ch;
		
	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull()&&entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_DI) {
		results[0] = start;
		results[1] = end;
		return true;
	}
		
	return false;
}
	
/**
 * ㅅ 불규칙 원형을 복원한다. (긋다--그어)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreSIrregular(const String& _start, const String& end,
									  /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(start.equals(_T(""))) return false;
		
	TCHAR ch = start.charAt(start.length()-1);
	TCHAR jasos[3];
	int jasos_length = MorphUtil::decompose(ch, jasos);
	if(jasos_length!=2) return false;
		
	ch = MorphUtil::makeChar(ch, 19);
	if(start.length()>1) 
		start = start.substring(0,start.length()-1)+ch;
	else
		start = ch;
		
	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull()&&entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_SIUT)	{
		results[0] = start;
		results[1] = end;
		return true;
	}

	return false;
}

/**
 * 르 불규칙 원형을 복원한다. (흐르다-->흘러)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreLIrregular(const String& start, const String& end,
									  /*out*/ String results[]) throw(MorphException) {
	if(start.length()<2) return false;
		
	TCHAR ch1 = start.charAt(start.length()-2);
	TCHAR ch2 = start.charAt(start.length()-1);
		
	TCHAR jasos1[3];
	int jasos1_length = MorphUtil::decompose(ch1, jasos1);
		

	if(jasos1_length==3&&jasos1[2]==_T('ㄹ')&&(ch2==_T('러')||ch2==_T('라'))) {
	
		String sb;
			
		ch1 = MorphUtil::makeChar(ch1, 0);
		if(start.length()>2) 
			sb.append(start.substring(0,start.length()-2)).append(ch1).append(_T("르"));
		else
			sb.append(ch1).append(_T("르"));

		DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(sb);
		if(!entry.isNull()&&entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_LOO) {
			results[0] = sb;
			results[1] = end;			
			return true;
		}			
	}
		
	return false;
}
	
/**
 * ㄹ불규칙 원형을 복원한다. (길다-->긴, 알다-->안)
 * 어간의 끝소리인 ‘ㄹ’이 ‘ㄴ’, ‘ㄹ’, ‘ㅂ’, ‘오’, ‘시’ 앞에서 탈락하는 활용의 형식
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreELIrregular(const String& _start, const String& end,
									   /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(end.length()==0) return false;
	        
	if(!(end.charAt(0)==_T('ㄴ')||end.charAt(0)==_T('ㄹ')||end.charAt(0)==_T('ㅂ')||end.charAt(0)==_T('오')||end.charAt(0)==_T('시'))) return false;
	    
	TCHAR convEnd = MorphUtil::makeChar(start.charAt(start.length()-1), 8);
	start = start.substring(0,start.length()-1)+convEnd;

	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull())	{
		results[0] = start;
		results[1] = end;
		return true;
	}	
		
	return false;
}
	
/**
 * 러 불규칙 원형을 복원한다. (이르다->이르러, 푸르다->푸르러)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreRUIrregular(const String& _start, const String& end,
									   /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(start.length()<2) return false;
		
	TCHAR ch1 = start.charAt(start.length()-1);
	TCHAR ch2 = start.charAt(start.length()-2);
		
	TCHAR jasos1[3];
	int jasos1_length = MorphUtil::decompose(ch1, jasos1);
	TCHAR jasos2[3];
	int jasos2_length = MorphUtil::decompose(ch2, jasos2);
	if(jasos1[0]!=_T('ㄹ')||jasos2[0]!=_T('ㄹ')) return false;
		
	ch2 = MorphUtil::makeChar(ch2, 0);
	if(start.length()>2) 
		start = start.substring(0,start.length()-1);
	else
		start = ch2;

	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull()&&entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_RU) {
		results[0] = start;
		results[1] = end;
		return true;
	}
		
	return false;
}
	
/**
 * ㅎ 탈락 원형을 복원한다. (까맣다-->까만,까매서)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreHIrregular(const String& _start, const String& end,
									  /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(start.equals(_T(""))||end.equals(_T(""))) return false;
	TCHAR ch1 = end.charAt(0);
	TCHAR ch2 = start.charAt(start.length()-1);

	TCHAR jasos1[3];
	int jasos1_length = MorphUtil::decompose(ch1, jasos1);
	TCHAR jasos2[3];
	int jasos2_length = MorphUtil::decompose(ch2, jasos2);
		
	if(jasos1_length==1) {
		ch2 = MorphUtil::makeChar(ch2, 27);
	}else {
		if(jasos2_length!=2||jasos2[1]!=_T('ㅐ')) return false;
		ch2 = MorphUtil::makeChar(ch2, 0, 27);
	}
						
	if(start.length()>1) 
		start = start.substring(0,start.length()-1)+ch2;
	else
		start = ch2;

	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull()&&entry->getFeature(WordEntry::IDX_REGURA)==IRR_TYPE_HIOOT) {
		results[0] = start;
		results[1] = end;
		return true;
	}
		
	return false;
}	

/**
 * 으 탈락 원형을 복원한다. (뜨다->더, 크다-커)
 * @param start
 * @param end
 * @return
 * @throws MorphException
*/
bool IrregularUtil::restoreUIrregular(const String& _start, const String& end,
									  /*out*/ String results[]) throw(MorphException) {
	String start = _start;
	if(start.equals(_T(""))) return false;	
	TCHAR ch = start.charAt(start.length()-1);		
	TCHAR jasos[3];
	int jasos_length = MorphUtil::decompose(ch, jasos);
	if(!(jasos_length==2&&jasos[1]==_T('ㅓ'))) return false;
		
	ch = MorphUtil::makeChar(ch, 18,0);

	if(start.length()>1) 
		start = start.substring(0,start.length()-1)+ch;
	else
		start = ch;

	DictionaryUtil::EntryPosition entry = DictionaryUtil::findVerb(start);
	if(!entry.isNull())	{
		results[0] = start;
		results[1] = end;
		return true;
	}
	
	return false;
}	

CL_NS_END2
