/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_utils_NounUtil
#define _lucene_analysis_kr_utils_NounUtil

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/StdHeader.h"
#include "../morph/AnalysisOutput.h"
#include "../morph/WordEntry.h"
#include "../morph/MorphException.h"
#include "String.h"

CL_NS_DEF2(analysis, kr)

class NounUtil {
    
public:
	/**
	 * 
	 * 어간부가 음/기 로 끝나는 경우
	 * 
	 * @param o
	 * @param candidates
	 * @return
	 * @throws MorphException
	 */
	static bool analysisMJ(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);

	  /**
	   * 용언 + '음/기' + 조사(PTN_VMXMJ)
	   * @param o
	   * @param candidates
	   * @return
	   * @throws MorphException
	   */
	static bool analysisVMJ(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
	  
	  /**
	   * 용언 + '아/어' + 보조용언 + '음/기' + 조사(PTN_VMXMJ)
	   * @param o
	   * @param candidates
	   * @return
	   * @throws MorphException
	   */
	static bool analysisVMXMJ(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
	  
	  /**
	   * 체언 + 용언화접미사 + '음/기' + 조사 (PTN_NSMJ)
	   * @param o
	   * @param candidates
	   * @return
	   * @throws MorphException
	   */
	static bool analysisNSMJ(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
	   
	  /**
	   * 복합명사인지 조사하고, 복합명사이면 단위명사들을 찾는다.
	   * 복합명사인지 여부는 단위명사가 모두 사전에 있는지 여부로 판단한다.
	   * 단위명사는 2글자 이상 단어에서만 찾는다.
	   * @param o
	   * @return
	   * @throws MorphException
	   */
	static bool confirmCNoun(AnalysisOutput& o) throw(MorphException);

	/*
	 * 마지막 음절이 명사형 접미사(등,상..)인지 조사한다.
	 */
	static void confirmDNoun(AnalysisOutput& output) throw(MorphException);
	    
	static bool endsWith2Josa(const String& input) throw(MorphException);

	
private:
	   /**
	    * 복합명사에서 단위명사를 분리해낸다.
	    * 리스트의 가장 마지막에 위치한 단어가 최장단어이다.
	    * @param str	복합명사
	    * @param pos
	    * @param o		분석결과
	    * @return		단위명사 리스트
	    * @throws MorphException
	    */
	static bool findNouns(const String& str, int pos, AnalysisOutput& o,
						  /*out*/ WordEntryList& nList) throw(MorphException);

	static bool isDNoun(const String &s) {
		static const String dnouns[] = { _T("등"), _T("들"), _T("상"), _T("간"), _T("") };
		for (int i = 0; !dnouns[i].empty(); i++) {
			if (dnouns[i].equals(s)) return true;
		}
		return false;
	}
};

CL_NS_END2
#endif
