/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "VerbUtil.h"
#include "EomiUtil.h"
#include "NounUtil.h"
#include "MorphUtil.h"
#include "IrregularUtil.h"
#include "DictionaryUtil.h"
#include "../morph/PatternConstants.h"

CL_NS_DEF2(analysis, kr)

const TCHAR* VerbUtil::verbSuffix[] = {
	_T("이"),_T("하"),_T("되"),_T("스럽"),_T("시키"),_T("있"),_T("없"),_T("같"),_T("당하"),_T("만하"),_T("드리"),_T("받"), NULL
	};
	
const TCHAR* VerbUtil::XVerb[] = {
	_T("오"),_T("내"),_T("주"),_T("보"),_T("지"),_T("오르"), NULL
};
	
   
/**
 * 3. 학교에서이다 : 체언 + '에서/부터/에서부터' + _T('이') + 어미 (PTN_NJCM) <br>
 */
bool VerbUtil::analysisNJCM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException) {
 
	int strlen = o.getStem().length();
	bool success = false;
	   
	if(strlen>3&&(o.getStem().endsWith(_T("에서이"))||o.getStem().endsWith(_T("부터이")))) {
		o.addElist(o.getStem().substring(strlen-1));
		o.setJosa(o.getStem().substring(strlen-3,strlen-1));
		o.setStem(o.getStem().substring(0,strlen-3));
		success = true;
	}else if(strlen>5&&(o.getStem().endsWith(_T("에서부터이")))) {
		o.addElist(o.getStem().substring(strlen-1));
		o.setJosa(o.getStem().substring(strlen-5,strlen-1));
		o.setStem(o.getStem().substring(0,strlen-5));
		success = true;
	}
	if(!success) return false;
	   
	if(success && !DictionaryUtil::findNoun(o.getStem()).isNull()) {		   
		o.setScore(AnalysisOutput::SCORE_CORRECT);
	}else {
		NounUtil::confirmCNoun(o);
	}
	   
	o.setPatn(PatternConstants::PTN_NJCM);
	o.setPos(PatternConstants::POS_NOUN);	
	candidates.push_back(o);
	   
	return true;
}
   
bool VerbUtil::analysisNSM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException) {
   
	int idxVbSfix = VerbUtil::endsWithVerbSuffix(o.getStem());
	if(idxVbSfix<1) return false;
		
	o.setVsfx(o.getStem().substring(idxVbSfix));
	o.setStem(o.getStem().substring(0,idxVbSfix));
	o.setPatn(PatternConstants::PTN_NSM);
	o.setPos(PatternConstants::POS_NOUN);
	
	DictionaryUtil::EntryPosition entry = DictionaryUtil::findNoun(o.getStem());
	if(entry.isNull() && NounUtil::confirmCNoun(o) && o.getCNounList().size() > 0) {
		entry = DictionaryUtil::findNoun(
			o.getCNounList().at(o.getCNounList().size()-1).getWord());
	}
	if(entry.isNull()) return false;
//		if(entry==null) {
//			NounUtil::confirmDNoun(o);
//			if(o.getScore()!=AnalysisOutput::SCORE_CORRECT) return false;
//		}
			
	if(o.getVsfx().equals(_T("하"))&&entry->getFeature(WordEntry::IDX_DOV)!=_T('1')) return false;
	if(o.getVsfx().equals(_T("되"))&&entry->getFeature(WordEntry::IDX_BEV)!=_T('1')) return false;				
	o.setScore(AnalysisOutput::SCORE_CORRECT);

	candidates.push_back(o);
	return true;

}
   
bool VerbUtil::analysisNSMXM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException) {
   
	int idxXVerb = VerbUtil::endsWithXVerb(o.getStem());
	if(idxXVerb==-1) return false;
		
	String eogan = o.getStem().substring(0,idxXVerb);
	String stomis[2];
		
	if((eogan.endsWith(_T("아"))||eogan.endsWith(_T("어")))&&eogan.length()>1)
		EomiUtil::splitEomi(eogan.substring(0,eogan.length()-1),
							eogan.substring(eogan.length()-1),
							stomis);
	else
		EomiUtil::splitEomi(eogan,_T(""),stomis);

	if(stomis[0].empty()) return false;
		
	o.addElist(stomis[1]);
	int idxVbSfix = VerbUtil::endsWithVerbSuffix(stomis[0]);
	if(idxVbSfix==-1) return false;
		
	o.setXverb(o.getStem().substring(idxXVerb));
	o.setVsfx(stomis[0].substring(idxVbSfix));
	o.setStem(stomis[0].substring(0,idxVbSfix));
	o.setPatn(PatternConstants::PTN_NSMXM);
	o.setPos(PatternConstants::POS_NOUN);
	DictionaryUtil::EntryPosition entry = DictionaryUtil::findNoun(o.getStem());
//		if(entry==null&&NounUtil::confirmCNoun(o)&&o.getCNounList().size()>0)	{
//			entry = DictionaryUtil::findNoun(o.getCNounList().get(o.getCNounList().size()-1));
//		}
	if(entry.isNull()) return false;	
		
	if(o.getVsfx().equals(_T("하"))&&entry->getFeature(WordEntry::IDX_DOV)!=_T('1')) return false;
	if(o.getVsfx().equals(_T("되"))&&entry->getFeature(WordEntry::IDX_BEV)!=_T('1')) return false;				
	o.setScore(AnalysisOutput::SCORE_CORRECT);
		
	candidates.push_back(o);						

		
	return true;	   
}
   
bool VerbUtil::analysisVMCM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException) {
   
	int strlen = o.getStem().length();
	   
	if(strlen<2) return false;
	   
	if(!o.getStem().endsWith(_T("이"))) return false;
	   
	TCHAR chrs[3];
	int chrs_length = MorphUtil::decompose(o.getStem().charAt(strlen-2), chrs);
	bool success = false;
	
	if(strlen>2&&o.getStem().endsWith(_T("기이"))) {
		o.setStem(o.getStem().substring(0,strlen-2));
		o.addElist(_T("기"));	   
		success = true;		   
	} else if(chrs_length>2&&chrs[2]==_T('ㅁ')){
		String eres[2];
		EomiUtil::splitEomi(o.getStem().substring(0,strlen-1), _T(""), eres);
		if (eres[0].empty()) return false;
			
		o.addElist(eres[1]);		   
		String irrs[2];		   
		if (IrregularUtil::restoreIrregularVerb(eres[0], eres[1], irrs)) o.setStem(irrs[0]);
		else o.setStem(eres[0]);

		success = true;
	}
	   
	if(success) {			   
		o.addElist(_T("이"));
		if(!DictionaryUtil::findVerb(o.getStem()).isNull()) {
			o.setPos(PatternConstants::POS_VERB);
			o.setPatn(PatternConstants::PTN_VMCM);
			o.setScore(AnalysisOutput::SCORE_CORRECT);
			candidates.push_back(o);
			return true;
		}		   
	}
	   
	return false;
	   
}
   
/**
 * 
 * 6. 도와주다 : 용언 + '아/어' + 보조용언 + 어미 (PTN_VMXM)
 * 
 * @param o
 * @param candidates
 * @return
 * @throws MorphException
 */
bool VerbUtil::analysisVMXM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException) {

	int idxXVerb = VerbUtil::endsWithXVerb(o.getStem());

	if(idxXVerb==-1) return false;
			
	o.setXverb(o.getStem().substring(idxXVerb));
		
	String eogan = o.getStem().substring(0,idxXVerb);

	String stomis[2];
	if(eogan.endsWith(_T("아"))||eogan.endsWith(_T("어"))) {
		EomiUtil::splitEomi(eogan.substring(0,eogan.length()-1),
							eogan.substring(eogan.length()-1),
							stomis);
		if (stomis[0].empty()) return false;
	}
	else {
		EomiUtil::splitEomi(eogan, _T(""), stomis);			
		if (stomis[0].empty() || !(stomis[1].startsWith(_T("아"))||stomis[1].startsWith(_T("어"))) )
			return false;
	}

	String irrs[2];
	if (IrregularUtil::restoreIrregularVerb(stomis[0], stomis[1], irrs)) {
		o.setStem(irrs[0]);
		o.addElist(irrs[1]);
	} else {
		o.setStem(stomis[0]);
		o.addElist(stomis[1]);
	}

	if (!DictionaryUtil::findVerb(o.getStem()).isNull()) {
		o.setPos(PatternConstants::POS_VERB);
		o.setPatn(PatternConstants::PTN_VMXM);
		o.setScore(AnalysisOutput::SCORE_CORRECT);
		candidates.push_back(o);
		return true;
	}	

	return false;	   
}


CL_NS_END2
