/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _lucene_analysis_kr_morph_VerbUtil
#define _lucene_analysis_kr_morph_VerbUtil

#if defined(_LUCENE_PRAGMA_ONCE)
# pragma once
#endif

#include "CLucene/StdHeader.h"
#include "../morph/AnalysisOutput.h"
#include "../morph/MorphException.h"
#include "MorphUtil.h"
#include "String.h"

CL_NS_DEF2(analysis, kr)


class VerbUtil {
public:
	static const TCHAR* verbSuffix[];
	
	static const TCHAR* XVerb[];
	
	/**
	 * 어간이 용언화접미사로 끝나면 index 를 반환한다.  아니면 -1을 반환한다.
	 * @param result
	 * @return
	 */
   static int endsWithVerbSuffix(const String& stem) {
	    int len = stem.length();
		for(int i = 0; verbSuffix[i] != NULL; i++) {
			if(stem.endsWith(verbSuffix[i])) return (len - _tcslen(verbSuffix[i]));
		}
		
		return -1;
   }
   
   /**
    * 어간부에 보조용언 [하,되,오,내,주,지]가 있는지 조사한다.
    * @param stem
    * @return
    */
   static int endsWithXVerb(const String& stem) {
	    int len = stem.length();
		for(int i = 0; XVerb[i] != NULL; i++) {
			if(stem.endsWith(XVerb[i])) return (len - _tcslen(XVerb[i]));
		}
	   return -1;
   }
   
   static bool constraintVerb(const String& start, const String& end) {
	   
	   TCHAR schs[3];
	   int schs_length = MorphUtil::decompose(start.charAt(start.length()-1), schs);
	   TCHAR echs[3];
	   int echs_length = MorphUtil::decompose(end.charAt(0), echs);
	   
	   if(schs_length==3 && schs[2]==_T('ㄹ') && echs[0]==_T('ㄹ')) return false;
	   
	   return true;
   }
   
   /**
    * 3. 학교에서이다 : 체언 + '에서/부터/에서부터' + _T('이') + 어미 (PTN_NJCM) <br>
    */
	static bool analysisNJCM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
    
	static bool analysisNSM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
      
	static bool analysisNSMXM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
	
	static bool analysisVMCM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);
   
   
   /**
    * 
    * 6. 도와주다 : 용언 + '아/어' + 보조용언 + 어미 (PTN_VMXM)
    * 
    * @param o
    * @param candidates
    * @return
    * @throws MorphException
    */
	static bool analysisVMXM(AnalysisOutput o, AnalysisOutputList& candidates) throw(MorphException);

};


CL_NS_END2
#endif
